@@ -25,6 +25,8 @@ import xml.sax as sax
 import os
 import io
 import shutil
+import copy
+from collections import OrderedDict
 
 from firewall import config
 from firewall.functions import u2b_if_py2
@@ -37,16 +39,17 @@ from firewall.errors import FirewallError
 
 class Service(IO_Object):
     IMPORT_EXPORT_STRUCTURE = (
-        ( "version",  "" ),                   # s
-        ( "short", "" ),                      # s
-        ( "description", "" ),                # s
-        ( "ports", [ ( "", "" ), ], ),        # a(ss)
-        ( "modules", [ "", ], ),              # as
-        ( "destination", { "": "", }, ),      # a{ss}
-        ( "protocols", [ "", ], ),            # as
-        ( "source_ports", [ ( "", "" ), ], ), # a(ss)
+        ( "version",  "" ),
+        ( "short", "" ),
+        ( "description", "" ),
+        ( "ports", [ ( "", "" ), ], ),
+        ( "modules", [ "", ], ),
+        ( "destination", { "": "", }, ),
+        ( "protocols", [ "", ], ),
+        ( "source_ports", [ ( "", "" ), ], ),
+        ( "includes", [ "" ], ),
+        ( "helpers", [ "", ], ),
         )
-    DBUS_SIGNATURE = '(sssa(ss)asa{ss}asa(ss))'
     ADDITIONAL_ALNUM_CHARS = [ "_", "-" ]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "short": None,
@@ -60,6 +63,8 @@ class Service(IO_Object):
         "module": [ "name" ],
         "destination": [ "ipv4", "ipv6" ],
         "source-port": [ "port", "protocol" ],
+        "include": [ "service" ],
+        "helper": [ "name" ],
         }
 
     def __init__(self):
@@ -72,6 +77,36 @@ class Service(IO_Object):
         self.modules = [ ]
         self.destination = { }
         self.source_ports = [ ]
+        self.includes = [ ]
+        self.helpers = [ ]
+
+    def import_config(self, conf):
+        self.check_config(conf)
+
+        for key in conf:
+            if not hasattr(self, key):
+                raise FirewallError(errors.UNKNOWN_ERROR, "Internal error. '{}' is not a valid attribute".format(key))
+            if isinstance(conf[key], list):
+                # maintain list order while removing duplicates
+                setattr(self, key, list(OrderedDict.fromkeys(copy.deepcopy(conf[key]))))
+            else:
+                setattr(self, key, copy.deepcopy(conf[key]))
+
+    def export_config(self):
+        conf = {}
+        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
+        for key in type_formats:
+            if getattr(self, key) or isinstance(getattr(self, key), bool):
+                conf[key] = copy.deepcopy(getattr(self, key))
+        return conf
+
+    def check_config(self, conf):
+        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
+        for key in conf:
+            if key not in [x for (x,y) in self.IMPORT_EXPORT_STRUCTURE]:
+                raise FirewallError(errors.INVALID_OPTION, "service option '{}' is not valid".format(key))
+            self._check_config_structure(conf[key], type_formats[key])
+            self._check_config(conf[key], key)
 
     def cleanup(self):
         self.version = ""
@@ -82,6 +117,8 @@ class Service(IO_Object):
         del self.modules[:]
         self.destination.clear()
         del self.source_ports[:]
+        del self.includes[:]
+        del self.helpers[:]
 
     def encode_strings(self):
         """ HACK. I haven't been able to make sax parser return
@@ -96,6 +133,8 @@ class Service(IO_Object):
         self.protocols = [u2b_if_py2(pr) for pr in self.protocols]
         self.source_ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr)
                              in self.source_ports]
+        self.includes = [u2b_if_py2(s) for s in self.includes]
+        self.helpers = [u2b_if_py2(s) for s in self.helpers]
 
     def _check_config(self, config, item):
         if item == "ports":
@@ -202,6 +241,18 @@ class service_ContentHandler(IO_Object_ContentHandler):
             else:
                 log.warning("Module '%s' already set, ignoring.",
                             module)
+        elif name == "include":
+            if attrs["service"] not in self.item.includes:
+                self.item.includes.append(attrs["service"])
+            else:
+                log.warning("Include '%s' already set, ignoring.",
+                            attrs["service"])
+        elif name == "helper":
+            if attrs["name"] not in self.item.helpers:
+                self.item.helpers.append(attrs["name"])
+            else:
+                log.warning("Helper '%s' already set, ignoring.",
+                            attrs["name"])
 
 
 def service_reader(filename, path):
@@ -312,6 +363,18 @@ def service_writer(service, path=None):
         handler.simpleElement("destination", service.destination)
         handler.ignorableWhitespace("\n")
 
+    # includes
+    for include in service.includes:
+        handler.ignorableWhitespace("  ")
+        handler.simpleElement("include", { "service": include })
+        handler.ignorableWhitespace("\n")
+
+    # helpers
+    for helper in service.helpers:
+        handler.ignorableWhitespace("  ")
+        handler.simpleElement("helper", { "name": helper })
+        handler.ignorableWhitespace("\n")
+
     # end service element
     handler.endElement('service')
     handler.ignorableWhitespace("\n")
